@@ -15,7 +15,7 @@ protects the user from the ever changing internal representation used in
 different versions of Python.
 """
 
-# Copyright (C) 2019-2021 Salvo "LtWorf" Tomaselli
+# Copyright (C) 2019-2022 Salvo "LtWorf" Tomaselli
 #
 # typedload is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -54,6 +54,8 @@ __all__ = [
     'is_typeddict',
     'is_newtype',
     'is_optional',
+    'is_notrequired',
+    'notrequiredtype',
     'uniontypes',
     'literalvalues',
     'NONETYPE',
@@ -85,6 +87,13 @@ except:
     pass
 
 
+try:
+    # Since 3.11
+    from typing import NotRequired  # type: ignore
+except ImportError:
+    NotRequired = None
+
+
 def _issubclass(t1, t2) -> bool:
     """
     Wrapper around _issubclass to circumvent python 3.7 changing API
@@ -278,3 +287,47 @@ def is_any(type_: Type[Any]) -> bool:
     Check if it is a typing.Any
     '''
     return type_ == Any
+
+
+def is_notrequired(type_: Type[Any]) -> bool:
+    '''
+    Check if it's typing.NotRequired or typing_extensions.NotRequired
+    '''
+    return getattr(type_, '__origin__', None) == NotRequired and NotRequired is not None
+
+
+def notrequiredtype(type_: Type[Any]) -> Type[Any]:
+    '''
+    Return the type wrapped by NotRequired
+    '''
+    return type_.__args__[0]
+
+
+if sys.version_info >= (3, 8):
+    def discriminatorliterals(type_: Type[Any]) -> Dict[str, Set[Any]]:
+        """
+        Takes an object type (NamedTuple, TypedDict, attrs, dataclass)
+        and returns which fields take a literal and which values are
+        allowed by the literal.
+
+        For unknown types, an empty dictionary is returned.
+
+        Since Literal exists only since 3.8, for previous versions
+        this always returns an empty dictionary
+        """
+
+        # Give up if the object is unknown
+        try:
+            d = type_.__annotations__.items()
+        except AttributeError:
+            return {}
+
+        r = {}
+        for k, v in d:
+            if not is_literal(v):
+                continue
+            r[k] = literalvalues(v)
+        return r
+else:
+   def discriminatorliterals(type_: Type[Any]) -> Dict[str, Set[Any]]:
+       return {}
